package com.bcxin.sync.task;

import com.bcxin.sync.service.tenant.TlkHardwareService;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * description：同步华为云设备定时任务
 * author：linchunpeng
 * date：2025/5/28
 */
@Slf4j
@Component
public class SyncHwDeviceTask {

    @Autowired
    private TlkHardwareService tlkHardwareService;
    @Autowired
    private RedissonClient redissonClient;

    //每5分钟执行
    @Scheduled(cron = "0 */10 * * * ?")
    public void syncHwDeviceTask() {
        log.info("===================================同步华为云设备定时任务===================================");
        //分布式锁key
        String lockKey = "REDISSON_LOCK_SYNC_HW_DEVICE_TASK";
        //取锁
        log.info("企业开通定时任务， lockKey：{}，取锁中.....", lockKey);
        RLock lock = redissonClient.getLock(lockKey);
        try {
            //加锁，并设置过期时间 300s
            if (lock.tryLock(2, 600, TimeUnit.SECONDS)) {
                log.info("取到锁");
                tlkHardwareService.syncAllHwDevice();
            } else {
                log.info("没有取到锁，跳过");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("同步华为云设备定时任务异常，{}", e.getMessage(), e);
        } finally {
            if (lock.isLocked() && lock.isHeldByCurrentThread()) {
                lock.unlock();
            }
            log.info("同步华为云设备定时任务，lockKey：{}，解锁", lockKey);
        }
        log.info("===================================同步华为云设备定时任务结束===================================");
    }

}
